@@ -45,9 +45,7 @@ shishi_as (Shishi * handle, Shishi_as ** as)
   Shishi_as *las;
   int res;
 
-  *as = malloc (sizeof (**as));
-  if (*as == NULL)
-    return SHISHI_MALLOC_ERROR;
+  *as = xmalloc (sizeof (**as));
   las = *as;
   memset (las, 0, sizeof (*las));
 
@@ -57,7 +55,7 @@ shishi_as (Shishi * handle, Shishi_as ** as)
   if (las->asreq == NULL)
     {
       shishi_error_printf (handle, "Could not create AS-REQ: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -70,7 +68,7 @@ shishi_as (Shishi * handle, Shishi_as ** as)
   if (las->asrep == NULL)
     {
       shishi_error_printf (handle, "Could not create AS-REP: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -78,7 +76,7 @@ shishi_as (Shishi * handle, Shishi_as ** as)
   if (las->krberror == NULL)
     {
       shishi_error_printf (handle, "Could not create KRB-ERROR: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -93,6 +91,24 @@ shishi_as (Shishi * handle, Shishi_as ** as)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_as_done:
+ * @as: structure that holds information about AS exchange
+ *
+ * Deallocate resources associated with AS exchange.  This should be
+ * called by the application when it no longer need to utilize the AS
+ * exchange handle.
+ **/
+void
+shishi_as_done (Shishi_as * as)
+{
+  shishi_asn1_done (as->handle, as->asreq);
+  shishi_asn1_done (as->handle, as->asrep);
+  shishi_asn1_done (as->handle, as->krberror);
+  shishi_tkt_done (as->tkt);
+  free (as);
+}
+
 /* TODO: add shishi_as_clientserver(h,p,a,client,server) and make the
    shishi_as_cnamerealmsname function take real cname/sname pointer
    arrays. */
@@ -157,7 +173,7 @@ shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_req_der (Shishi_as * as, char **out, size_t *outlen)
+shishi_as_req_der (Shishi_as * as, char **out, size_t * outlen)
 {
   int rc;
 
@@ -242,7 +258,7 @@ shishi_as_rep_process (Shishi_as * as, Shishi_key * key, const char *password)
     {
       shishi_error_printf (as->handle, "Could not extract cname and "
 			   "realm from AS-REQ: %s\n", shishi_strerror (res),
-			   shishi_strerror_details (as->handle));
+			   shishi_error (as->handle));
       return res;
     }
   user[userlen] = '\0';
@@ -285,7 +301,7 @@ shishi_as_rep_process (Shishi_as * as, Shishi_key * key, const char *password)
     {
       shishi_error_printf (as->handle,
 			   "Could not extract ticket from AS-REP: %s",
-			   shishi_strerror_details (as->handle));
+			   shishi_error (as->handle));
       return res;
     }
 
@@ -295,12 +311,8 @@ shishi_as_rep_process (Shishi_as * as, Shishi_key * key, const char *password)
   if (VERBOSEASN1 (as->handle))
     shishi_ticket_print (as->handle, stdout, ticket);
 
+  /* XXX */
   as->tkt = shishi_tkt2 (as->handle, ticket, kdcreppart, as->asrep);
-  if (as->tkt == NULL)
-    {
-      shishi_error_printf (as->handle, "Could not create ticket");
-      return SHISHI_MALLOC_ERROR;
-    }
 
   return SHISHI_OK;
 }
@@ -373,7 +385,7 @@ shishi_as_rep_build (Shishi_as * as, Shishi_key * key)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_rep_der (Shishi_as * as, char **out, size_t *outlen)
+shishi_as_rep_der (Shishi_as * as, char **out, size_t * outlen)
 {
   int rc;
 
@@ -450,7 +462,7 @@ shishi_as_krberror (Shishi_as * as)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_krberror_der (Shishi_as * as, char **out, size_t *outlen)
+shishi_as_krberror_der (Shishi_as * as, char **out, size_t * outlen)
 {
   int rc;
 
