@@ -75,7 +75,8 @@ shishi_der2asn1 (Shishi * handle,
 
 int
 shishi_a2d_field (Shishi * handle,
-		  Shishi_asn1 node, const char *field, char *der, int *len)
+		  Shishi_asn1 node, const char *field,
+		  char *der, size_t * len)
 {
   char errorDescription[MAX_ERROR_DESCRIPTION_SIZE] = "";
   int rc;
@@ -92,14 +93,14 @@ shishi_a2d_field (Shishi * handle,
 }
 
 int
-shishi_a2d (Shishi * handle, Shishi_asn1 node, char *der, int *len)
+shishi_a2d (Shishi * handle, Shishi_asn1 node, char *der, size_t * len)
 {
   return shishi_a2d_field (handle, node, "", der, len);
 }
 
 int
 shishi_a2d_new_field (Shishi * handle, Shishi_asn1 node,
-		      const char *field, char **der, size_t *len)
+		      const char *field, char **der, size_t * len)
 {
   char errorDescription[MAX_ERROR_DESCRIPTION_SIZE] = "";
   int rc;
@@ -112,9 +113,7 @@ shishi_a2d_new_field (Shishi * handle, Shishi_asn1 node,
       return SHISHI_ASN1_ERROR;
     }
 
-  *der = malloc (*len);
-  if (!*der)
-    return SHISHI_MALLOC_ERROR;
+  *der = xmalloc (*len);
 
   rc = asn1_der_coding (node, field, *der, len, errorDescription);
   if (rc != ASN1_SUCCESS)
@@ -127,12 +126,12 @@ shishi_a2d_new_field (Shishi * handle, Shishi_asn1 node,
 }
 
 int
-shishi_new_a2d (Shishi * handle, Shishi_asn1 node, char **der, size_t *len)
+shishi_new_a2d (Shishi * handle, Shishi_asn1 node, char **der, size_t * len)
 {
   return shishi_a2d_new_field (handle, node, "", der, len);
 }
 
-int
+void
 shishi_asn1_done (Shishi * handle, Shishi_asn1 node)
 {
 
@@ -140,12 +139,8 @@ shishi_asn1_done (Shishi * handle, Shishi_asn1 node)
 
   rc = asn1_delete_structure (&node);
   if (rc != ASN1_SUCCESS)
-    {
-      shishi_error_set (handle, libtasn1_strerror (rc));
-      return SHISHI_ASN1_ERROR;
-    }
-
-  return SHISHI_OK;
+    shishi_error_printf (handle, "Cannot dellocate ASN.1 structure: %s",
+			 libtasn1_strerror (rc));
 }
 
 int
@@ -390,11 +385,11 @@ shishi_asn1_read_empty_p (Shishi * handle,
 
 int
 shishi_asn1_number_of_elements (Shishi * handle, Shishi_asn1 node,
-				const char *field, int *n)
+				const char *field, size_t * n)
 {
   int rc;
 
-  rc = asn1_number_of_elements (node, field, n);
+  rc = asn1_number_of_elements (node, field, (int *) n);
   if (rc != ASN1_SUCCESS)
     {
       if (rc == ASN1_ELEMENT_NOT_FOUND)
@@ -531,6 +526,19 @@ shishi_asn1_krbsafe (Shishi * handle)
   return shishi_asn1_new (handle, "Kerberos5.KRB-SAFE", "KRB-SAFE");
 }
 
+Shishi_asn1
+shishi_asn1_priv (Shishi * handle)
+{
+  return shishi_asn1_new (handle, "Kerberos5.KRB-PRIV", "KRB-PRIV");
+}
+
+Shishi_asn1
+shishi_asn1_encprivpart (Shishi * handle)
+{
+  return shishi_asn1_new (handle, "Kerberos5.EncKrbPrivPart",
+			  "EncKrbPrivPart");
+}
+
 Shishi_asn1
 shishi_der2asn1_ticket (Shishi * handle, const char *der, size_t derlen)
 {
@@ -656,6 +664,20 @@ shishi_der2asn1_krbsafe (Shishi * handle, const char *der, size_t derlen)
 			  der, derlen);
 }
 
+Shishi_asn1
+shishi_der2asn1_priv (Shishi * handle, const char *der, size_t derlen)
+{
+  return shishi_der2asn1 (handle, "Kerberos5.KRB-PRIV", "KRB-PRIV",
+			  der, derlen);
+}
+
+Shishi_asn1
+shishi_der2asn1_encprivpart (Shishi * handle, const char *der, size_t derlen)
+{
+  return shishi_der2asn1 (handle, "Kerberos5.EncKrbPrivPart", "EncKrbPrivPart",
+			  der, derlen);
+}
+
 Shishi_asn1
 shishi_der2asn1_apreq (Shishi * handle, const char *der, size_t derlen)
 {
