@@ -71,7 +71,7 @@ des3none_dencrypt (Shishi * handle,
 	simplified_dencrypt (handle, derivedkey, iv, ivlen, ivout, ivoutlen,
 			     in, inlen, out, outlen, direction);
 
-      shishi_key_done (&derivedkey);
+      shishi_key_done (derivedkey);
 
       if (res != SHISHI_OK)
 	return res;
@@ -210,7 +210,7 @@ des3_string_to_key (Shishi * handle,
 
   /* s = passwordString + salt */
   n_s = stringlen + saltlen;
-  s = (char *) malloc (n_s);
+  s = (char *) xmalloc (n_s);
   memcpy (s, string, stringlen);
   memcpy (s + stringlen, salt, saltlen);
 
@@ -233,7 +233,7 @@ des3_string_to_key (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  shishi_key_done (&key);
+  shishi_key_done (key);
 
   if (VERBOSECRYPTO (handle))
     {
@@ -252,7 +252,7 @@ des3_checksum (Shishi * handle,
 	       Shishi_key * key,
 	       int keyusage,
 	       int cksumtype,
-	       char *in, size_t inlen, char **out, size_t * outlen)
+	       const char *in, size_t inlen, char **out, size_t * outlen)
 {
   return simplified_checksum (handle, key, keyusage, cksumtype,
 			      in, inlen, out, outlen);
