@@ -620,7 +620,7 @@ des_string_to_key (Shishi * handle,
   n_s = stringlen + saltlen;
   if ((n_s % 8) != 0)
     n_s += 8 - n_s % 8;
-  s = (char *) malloc (n_s);
+  s = (char *) xmalloc (n_s);
   memcpy (s, string, stringlen);
   if (saltlen > 0)
     memcpy (s + stringlen, salt, saltlen);
@@ -781,7 +781,8 @@ des_checksum (Shishi * handle,
 	      Shishi_key * key,
 	      int keyusage,
 	      int cksumtype,
-	      char *in, size_t inlen, char **out, size_t * outlen, int algo)
+	      const char *in, size_t inlen,
+	      char **out, size_t * outlen, int algo)
 {
   char cksum[8 + MAX_HASH_LEN];
   size_t cksumlen;
@@ -796,7 +797,6 @@ des_checksum (Shishi * handle,
       return res;
     }
 
-
   keyp = shishi_key_value (key);
 
   for (i = 0; i < 8; i++)
@@ -822,7 +822,7 @@ des_md4_checksum (Shishi * handle,
 		  Shishi_key * key,
 		  int keyusage,
 		  int cksumtype,
-		  char *in, size_t inlen, char **out, size_t * outlen)
+		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
   return des_checksum (handle, key, keyusage, cksumtype,
 		       in, inlen, out, outlen, SHISHI_DES_CBC_MD4);
@@ -833,7 +833,7 @@ des_md5_checksum (Shishi * handle,
 		  Shishi_key * key,
 		  int keyusage,
 		  int cksumtype,
-		  char *in, size_t inlen, char **out, size_t * outlen)
+		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
   return des_checksum (handle, key, keyusage, cksumtype,
 		       in, inlen, out, outlen, SHISHI_DES_CBC_MD5);
@@ -844,7 +844,7 @@ gss_des_checksum (Shishi * handle,
 		  Shishi_key * key,
 		  int keyusage,
 		  int cksumtype,
-		  char *in, size_t inlen, char **out, size_t * outlen)
+		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
 #ifdef USE_GCRYPT
   char *keyp;
@@ -911,3 +911,57 @@ gss_des_checksum (Shishi * handle,
 #endif
   return SHISHI_OK;
 }
+
+static int
+des_md5_verify (Shishi * handle,
+		Shishi_key * key,
+		int keyusage,
+		int cksumtype,
+		const char *in, size_t inlen,
+		const char *cksum, size_t cksumlen)
+{
+  struct md5_ctx md5;
+  char *out;
+  size_t outlen;
+  char md[MD5_DIGEST_SIZE];
+  char *keyp;
+  size_t i;
+  int res;
+
+  if (cksumlen != 8 + MD5_DIGEST_SIZE)
+    return SHISHI_VERIFY_FAILED;
+
+  /*
+   * get_mic                   des-cbc(key XOR 0xF0F0F0F0F0F0F0F0,
+   *                                   conf | rsa-md5(conf | msg))
+   * verify_mic                decrypt and verify rsa-md5 checksum
+   */
+
+  keyp = shishi_key_value (key);
+
+  for (i = 0; i < 8; i++)
+    keyp[i] ^= 0xF0;
+
+  res = simplified_decrypt (handle, key, 0, NULL, 0, NULL, NULL,
+			    cksum, cksumlen, &out, &outlen);
+
+  for (i = 0; i < 8; i++)
+    keyp[i] ^= 0xF0;
+
+  if (res != SHISHI_OK)
+    {
+      shishi_error_set (handle, "decrypt failed");
+      return res;
+    }
+
+  md5_init (&md5);
+  md5_update (&md5, 8, out);
+  md5_update (&md5, inlen, in);
+
+  md5_digest (&md5, MD5_DIGEST_SIZE, md);
+
+  if (memcmp (out + 8, md, MD5_DIGEST_SIZE) != 0)
+    return SHISHI_VERIFY_FAILED;
+
+  return SHISHI_OK;
+}
