@@ -19,8 +19,6 @@
  *
  */
 
-/* XXX several functions with out/outlen writes longer than the outlen */
-
 #include "internal.h"
 #ifdef USE_GCRYPT
 #include <gcrypt.h>
@@ -257,8 +255,7 @@ simplified_hmac (Shishi * handle,
     }
 
   *outhashlen = hlen;
-  *outhash = xmalloc (*outhashlen);
-  memcpy (*outhash, hash, *outhashlen);
+  *outhash = xmemdup (*outhash, hash, *outhashlen);
 
   gcry_md_close (mdh);
 #else
@@ -487,9 +484,8 @@ simplified_dencrypt (Shishi * handle,
       if (ivout && ivoutlen)
 	{
 	  *ivoutlen = sizeof (des.iv);
-	  *ivout = xmalloc (*ivoutlen);
 	  /* XXX see above */
-	  memcpy (*ivout, des.iv, *ivoutlen);
+	  *ivout = xmemdup (*ivout, des.iv, *ivoutlen);
 	}
       break;
 
@@ -511,9 +507,8 @@ simplified_dencrypt (Shishi * handle,
       if (ivout && ivoutlen)
 	{
 	  *ivoutlen = sizeof (des3.iv);
-	  *ivout = xmalloc (*ivoutlen);
 	  /* XXX see above */
-	  memcpy (*ivout, des3.iv, *ivoutlen);
+	  *ivout = xmemdup (*ivout, des3.iv, *ivoutlen);
 	}
       break;
 
@@ -537,9 +532,8 @@ simplified_dencrypt (Shishi * handle,
       if (ivout && ivoutlen)
 	{
 	  *ivoutlen = sizeof (aes.iv);
-	  *ivout = xmalloc (*ivoutlen);
 	  /* XXX see above */
-	  memcpy (*ivout, aes.iv, *ivoutlen);
+	  *ivout = xmemdup (*ivout, aes.iv, *ivoutlen);
 	}
       break;
     }
@@ -613,9 +607,9 @@ simplified_encrypt (Shishi * handle,
 
     done:
       if (&privacykey)
-	shishi_key_done (&privacykey);
+	shishi_key_done (privacykey);
       if (&integritykey)
-	shishi_key_done (&integritykey);
+	shishi_key_done (integritykey);
       if (hmac)
 	free (hmac);
       if (ct)
@@ -677,9 +671,9 @@ simplified_decrypt (Shishi * handle,
 
     done:
       if (privacykey)
-	shishi_key_done (&privacykey);
+	shishi_key_done (privacykey);
       if (integritykey)
-	shishi_key_done (&integritykey);
+	shishi_key_done (integritykey);
     }
   else
     {
@@ -695,7 +689,8 @@ simplified_checksum (Shishi * handle,
 		     Shishi_key * key,
 		     int keyusage,
 		     int cksumtype,
-		     char *in, size_t inlen, char **out, size_t * outlen)
+		     const char *in, size_t inlen,
+		     char **out, size_t * outlen)
 {
   Shishi_key *checksumkey;
   int cksumlen = shishi_checksum_cksumlen (cksumtype);
@@ -708,7 +703,7 @@ simplified_checksum (Shishi * handle,
 
   res = simplified_hmac (handle, checksumkey, in, inlen, out, outlen);
 
-  shishi_key_done (&checksumkey);
+  shishi_key_done (checksumkey);
 
   if (res != SHISHI_OK)
     return res;
@@ -770,13 +765,22 @@ typedef int (*Shishi_checksum_function) (Shishi * handle,
 					 Shishi_key * key,
 					 int keyusage,
 					 int cksumtype,
-					 char *in, size_t inlen,
+					 const char *in, size_t inlen,
 					 char **out, size_t * outlen);
 
+typedef int (*Shishi_verify_function) (Shishi * handle,
+				       Shishi_key * key,
+				       int keyusage,
+				       int cksumtype,
+				       const char *in, size_t inlen,
+				       const char *cksum, size_t cksumlen);
+
 #include "crypto-null.c"
+#include "crypto-md.c"
 #include "crypto-des.c"
 #include "crypto-3des.c"
 #include "crypto-aes.c"
+#include "crypto-rc4.c"
 
 struct cipherinfo
 {
@@ -796,7 +800,7 @@ struct cipherinfo
 typedef struct cipherinfo cipherinfo;
 
 static cipherinfo null_info = {
-  0,
+  SHISHI_NULL,
   "NULL",
   1,
   0,
@@ -811,7 +815,7 @@ static cipherinfo null_info = {
 };
 
 static cipherinfo des_cbc_crc_info = {
-  1,
+  SHISHI_DES_CBC_CRC,
   "des-cbc-crc",
   8,
   4,
@@ -826,7 +830,7 @@ static cipherinfo des_cbc_crc_info = {
 };
 
 static cipherinfo des_cbc_md4_info = {
-  2,
+  SHISHI_DES_CBC_MD4,
   "des-cbc-md4",
   8,
   0,
@@ -841,7 +845,7 @@ static cipherinfo des_cbc_md4_info = {
 };
 
 static cipherinfo des_cbc_md5_info = {
-  3,
+  SHISHI_DES_CBC_MD5,
   "des-cbc-md5",
   8,
   0,
@@ -856,7 +860,7 @@ static cipherinfo des_cbc_md5_info = {
 };
 
 static cipherinfo des_cbc_none_info = {
-  4,
+  SHISHI_DES_CBC_NONE,
   "des-cbc-none",
   8,
   0,
@@ -870,23 +874,8 @@ static cipherinfo des_cbc_none_info = {
   des_none_decrypt
 };
 
-static cipherinfo des3_cbc_sha1_kd_info = {
-  16,
-  "des3-cbc-sha1-kd",
-  8,
-  0,
-  8,
-  3 * 8,
-  3 * 8,
-  SHISHI_HMAC_SHA1_DES3_KD,
-  des3_random_to_key,
-  des3_string_to_key,
-  _des3_encrypt,
-  _des3_decrypt
-};
-
 static cipherinfo des3_cbc_none_info = {
-  6,
+  SHISHI_DES3_CBC_NONE,
   "des3-cbc-none",
   8,
   0,
@@ -900,8 +889,23 @@ static cipherinfo des3_cbc_none_info = {
   des3none_decrypt
 };
 
+static cipherinfo des3_cbc_sha1_kd_info = {
+  SHISHI_DES3_CBC_HMAC_SHA1_KD,
+  "des3-cbc-sha1-kd",
+  8,
+  0,
+  8,
+  3 * 8,
+  3 * 8,
+  SHISHI_HMAC_SHA1_DES3_KD,
+  des3_random_to_key,
+  des3_string_to_key,
+  _des3_encrypt,
+  _des3_decrypt
+};
+
 static cipherinfo aes128_cts_hmac_sha1_96_info = {
-  17,
+  SHISHI_AES128_CTS_HMAC_SHA1_96,
   "aes128-cts-hmac-sha1-96",
   16,
   0,
@@ -916,7 +920,7 @@ static cipherinfo aes128_cts_hmac_sha1_96_info = {
 };
 
 static cipherinfo aes256_cts_hmac_sha1_96_info = {
-  18,
+  SHISHI_AES256_CTS_HMAC_SHA1_96,
   "aes256-cts-hmac-sha1-96",
   16,
   0,
@@ -930,6 +934,36 @@ static cipherinfo aes256_cts_hmac_sha1_96_info = {
   aes256_decrypt
 };
 
+static cipherinfo rc4_hmac_info = {
+  SHISHI_RC4_HMAC,
+  "rc4-hmac",
+  16,
+  0,
+  16,
+  16,
+  16,
+  SHISHI_RC4_HMAC_MD5,
+  rc4_hmac_random_to_key,
+  rc4_hmac_string_to_key,
+  rc4_hmac_encrypt,
+  rc4_hmac_decrypt
+};
+
+static cipherinfo rc4_hmac_exp_info = {
+  SHISHI_RC4_HMAC_EXP,
+  "rc4-hmac-exp",
+  16,
+  0,
+  16,
+  16,
+  16,
+  SHISHI_RC4_HMAC_MD5,
+  rc4_hmac_random_to_key,
+  rc4_hmac_string_to_key,
+  rc4_hmac_exp_encrypt,
+  rc4_hmac_exp_decrypt
+};
+
 static cipherinfo *ciphers[] = {
   &null_info,
   &des_cbc_crc_info,
@@ -939,7 +973,9 @@ static cipherinfo *ciphers[] = {
   &des3_cbc_none_info,
   &des3_cbc_sha1_kd_info,
   &aes128_cts_hmac_sha1_96_info,
-  &aes256_cts_hmac_sha1_96_info
+  &aes256_cts_hmac_sha1_96_info,
+  &rc4_hmac_info,
+  &rc4_hmac_exp_info
 };
 
 /**
@@ -1175,10 +1211,18 @@ struct checksuminfo
   char *name;
   int cksumlen;
   Shishi_checksum_function checksum;
+  Shishi_verify_function verify;
 };
 typedef struct checksuminfo checksuminfo;
 
 static checksuminfo md4_info = {
+  SHISHI_RSA_MD4,
+  "rsa-md4",
+  16,
+  md4_checksum
+};
+
+static checksuminfo md4_des_info = {
   SHISHI_RSA_MD4_DES,
   "rsa-md4-des",
   24,
@@ -1186,10 +1230,18 @@ static checksuminfo md4_info = {
 };
 
 static checksuminfo md5_info = {
+  SHISHI_RSA_MD5,
+  "rsa-md5",
+  16,
+  md5_checksum
+};
+
+static checksuminfo md5_des_info = {
   SHISHI_RSA_MD5_DES,
   "rsa-md5-des",
   24,
-  des_md5_checksum
+  des_md5_checksum,
+  des_md5_verify
 };
 
 static checksuminfo md5_gss_info = {
@@ -1220,13 +1272,23 @@ static checksuminfo hmac_sha1_96_aes256_info = {
   aes256_checksum
 };
 
+static checksuminfo rc4_hmac_md5_info = {
+  SHISHI_RC4_HMAC_MD5,
+  "rc4-hmac-md5",
+  16,
+  rc4_hmac_md5_checksum
+};
+
 static checksuminfo *checksums[] = {
   &md4_info,
+  &md4_des_info,
   &md5_info,
+  &md5_des_info,
   &md5_gss_info,
   &hmac_sha1_des3_kd_info,
   &hmac_sha1_96_aes128_info,
-  &hmac_sha1_96_aes256_info
+  &hmac_sha1_96_aes256_info,
+  &rc4_hmac_md5_info
 };
 
 /**
@@ -1326,6 +1388,18 @@ _shishi_checksum (int32_t type)
   return NULL;
 }
 
+static Shishi_verify_function
+_shishi_verify (int32_t type)
+{
+  size_t i;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    if (type == checksums[i]->type)
+      return checksums[i]->verify;
+
+  return NULL;
+}
+
 /**
  * shishi_string_to_key:
  * @handle: shishi handle as allocated by shishi_init().
@@ -1452,20 +1526,17 @@ shishi_random_to_key (Shishi * handle,
 /**
  * shishi_checksum:
  * @handle: shishi handle as allocated by shishi_init().
- * @key: key to encrypt with.
- * @keyusage: integer specifying what this key is encrypting.
+ * @key: key to compute checksum with.
+ * @keyusage: integer specifying what this key is used for.
  * @cksumtype: the checksum algorithm to use.
  * @in: input array with data to integrity protect.
  * @inlen: size of input array with data to integrity protect.
- * @out: output array with integrity protected data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated integrity protected data.
+ * @outlen: output variable with length of output array with checksum.
  *
  * Integrity protect data using key, possibly altered by supplied key
- * usage.  If key usage is 0, no key derivation is used.
- *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * usage.  If key usage is 0, no key derivation is used.  The OUT
+ * buffer must be deallocated by the caller.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
@@ -1474,7 +1545,7 @@ shishi_checksum (Shishi * handle,
 		 Shishi_key * key,
 		 int keyusage,
 		 int cksumtype,
-		 char *in, size_t inlen, char **out, size_t * outlen)
+		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
   Shishi_checksum_function checksum;
   int res;
@@ -1502,8 +1573,10 @@ shishi_checksum (Shishi * handle,
       return SHISHI_CRYPTO_ERROR;
     }
 
-  res =
-    (*checksum) (handle, key, keyusage, cksumtype, in, inlen, out, outlen);
+  /* XXX? check if etype and cksumtype are compatible? */
+
+  res = (*checksum) (handle, key, keyusage, cksumtype,
+		     in, inlen, out, outlen);
 
   if (VERBOSECRYPTO (handle))
     {
@@ -1516,6 +1589,70 @@ shishi_checksum (Shishi * handle,
   return res;
 }
 
+/**
+ * shishi_verify:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to verify checksum with.
+ * @keyusage: integer specifying what this key is used for.
+ * @cksumtype: the checksum algorithm to use.
+ * @in: input array with data that was integrity protected.
+ * @inlen: size of input array with data that was integrity protected.
+ * @cksum: input array with alleged checksum of data.
+ * @cksumlen: size of input array with alleged checksum of data.
+ *
+ * Verify checksum of data using key, possibly altered by supplied key
+ * usage.  If key usage is 0, no key derivation is used.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_verify (Shishi * handle,
+	       Shishi_key * key,
+	       int keyusage,
+	       int cksumtype,
+	       const char *in, size_t inlen,
+	       const char *cksum, size_t cksumlen)
+{
+  Shishi_verify_function verify;
+  int res;
+
+  if (VERBOSECRYPTO (handle))
+    {
+      printf ("verify (%s, %d, in, out)\n", shishi_key_name (key), cksumtype);
+      printf ("\t ;; key (%d):\n", shishi_key_length (key));
+      hexprint (shishi_key_value (key), shishi_key_length (key));
+      puts ("");
+      printf ("\t ;; data:\n");
+      escapeprint (in, inlen);
+      hexprint (in, inlen);
+      puts ("");
+      printf ("\t ;; mic:\n");
+      escapeprint (cksum, cksumlen);
+      hexprint (cksum, cksumlen);
+      puts ("");
+    }
+
+  if (cksumtype == 0)
+    cksumtype = shishi_cipher_defaultcksumtype (shishi_key_type (key));
+
+  verify = _shishi_verify (cksumtype);
+  if (verify == NULL)
+    {
+      shishi_error_printf (handle, "Unsupported checksum type %d", cksumtype);
+      return SHISHI_CRYPTO_ERROR;
+    }
+
+  /* XXX? check if etype and cksumtype are compatible? */
+
+  res = (*verify) (handle, key, keyusage, cksumtype,
+		   in, inlen, cksum, cksumlen);
+
+  if (VERBOSECRYPTO (handle))
+    printf ("\t ;; verify return: %d\n", res);
+
+  return res;
+}
+
 /**
  * shishi_encrypt_ivupdate_etype:
  * @handle: shishi handle as allocated by shishi_init().
@@ -2177,7 +2314,7 @@ shishi_randomize (Shishi * handle, char *data, size_t datalen)
  **/
 int
 shishi_n_fold (Shishi * handle,
-	       char *in, size_t inlen, char *out, size_t outlen)
+	       const char *in, size_t inlen, char *out, size_t outlen)
 {
   int m = inlen;
   int n = outlen;
@@ -2195,10 +2332,7 @@ shishi_n_fold (Shishi * handle,
      to yield a n-bit result denoted <X>_n.
    */
 
-  a = (char *) malloc (m);
-  if (a == NULL)
-    return SHISHI_MALLOC_ERROR;
-  memcpy (a, in, m);
+  a = (char *) xmemdup (a, in, m);
 
   lcmmn = lcm (m, n);
 
@@ -2214,9 +2348,7 @@ shishi_n_fold (Shishi * handle,
       puts ("");
     }
 
-  buf = (char *) malloc (lcmmn);
-  if (buf == NULL)
-    return SHISHI_MALLOC_ERROR;
+  buf = (char *) xmalloc (lcmmn);
 
   /* Replicate the input th the LCMMN length */
   for (i = 0; i < (lcmmn / m); i++)
@@ -2318,7 +2450,7 @@ shishi_n_fold (Shishi * handle,
 int
 shishi_dr (Shishi * handle,
 	   Shishi_key * key,
-	   char *constant, size_t constantlen,
+	   const char *constant, size_t constantlen,
 	   char *derivedrandom, size_t derivedrandomlen)
 {
   char *cipher;
@@ -2332,10 +2464,10 @@ shishi_dr (Shishi * handle,
     {
       printf ("dr (%s, key, constant, %d)\n",
 	      shishi_cipher_name (shishi_key_type (key)), derivedrandomlen);
-      printf ("\t ;; key (length %d):\n", shishi_key_type (key));
-      hexprint (shishi_key_value (key), shishi_key_type (key));
+      printf ("\t ;; key (length %d):\n", shishi_key_length (key));
+      hexprint (shishi_key_value (key), shishi_key_length (key));
       puts ("");
-      binprint (shishi_key_value (key), shishi_key_type (key));
+      binprint (shishi_key_value (key), shishi_key_length (key));
       puts ("");
       printf ("\t ;; constant  %s':\n", constant);
       escapeprint (constant, constantlen);
@@ -2417,7 +2549,7 @@ shishi_dr (Shishi * handle,
 int
 shishi_dk (Shishi * handle,
 	   Shishi_key * key,
-	   char *constant, int constantlen, Shishi_key * derivedkey)
+	   const char *constant, size_t constantlen, Shishi_key * derivedkey)
 {
   char random[MAX_RANDOM_LEN];
   int res;
