@@ -21,9 +21,6 @@
 
 #include "internal.h"
 
-#define INFOSTR "libshishi: info: "
-#define WARNSTR "libshishi: warning: "
-
 
 /**
  * shishi:
@@ -42,13 +39,7 @@ shishi (void)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  handle = (Shishi *) malloc (sizeof (*handle));
-  if (!handle)
-    {
-      fprintf (stderr, "libshishi: error: %s\n",
-	       shishi_strerror (SHISHI_MALLOC_ERROR));
-      return NULL;
-    }
+  handle = (Shishi *) xmalloc (sizeof (*handle));
   memset ((void *) handle, 0, sizeof (*handle));
 
   if (_shishi_cipher_init () != SHISHI_OK)
@@ -68,13 +59,8 @@ shishi (void)
   handle->renewlife = RENEWLIFE;
 
   handle->nclientkdcetypes = 1;
-  handle->clientkdcetypes = malloc (sizeof (*handle->clientkdcetypes) *
-				    handle->nclientkdcetypes);
-  if (handle->clientkdcetypes == NULL)
-    {
-      shishi_warn (handle, "%s", shishi_strerror (SHISHI_MALLOC_ERROR));
-      return NULL;
-    }
+  handle->clientkdcetypes = xmalloc (sizeof (*handle->clientkdcetypes) *
+				     handle->nclientkdcetypes);
   handle->clientkdcetypes[0] = SHISHI_AES256_CTS_HMAC_SHA1_96;
 
   tmp = shishi_realm_default_guess ();
@@ -91,6 +77,44 @@ shishi (void)
   return handle;
 }
 
+/**
+ * shishi_done:
+ * @handle: shishi handle as allocated by shishi_init().
+ *
+ * Deallocates the shishi library handle.  The handle must not be used
+ * in any calls to shishi functions after this.  If there is a default
+ * tkts, it is written to the default tkts file (call
+ * shishi_tkts_default_file_set() to change the default tkts
+ * file). If you do not wish to write the default tkts file,
+ * close the default tkts with shishi_tkts_done(handle,
+ * NULL) before calling this function.
+ **/
+void
+shishi_done (Shishi * handle)
+{
+  if (handle->tkts)
+    {
+      shishi_tkts_to_file (handle->tkts, shishi_tkts_default_file (handle));
+
+      shishi_tkts_done (&handle->tkts);
+    }
+
+  /*  if (handle->default_realm)
+     free (handle->default_realm); */
+  if (handle->usercfgfile)
+    free (handle->usercfgfile);
+  if (handle->tktsdefaultfile)
+    free (handle->tktsdefaultfile);
+  if (handle->hostkeysdefaultfile)
+    free (handle->hostkeysdefaultfile);
+  if (handle->clientkdcetypes)
+    free (handle->clientkdcetypes);
+
+  if (handle->asn1)
+    shishi_asn1_done (handle, handle->asn1);
+
+  free (handle);
+}
 
 static void
 _shishi_maybe_install_usercfg (Shishi * handle)
@@ -311,29 +335,3 @@ shishi_init_server_with_paths (Shishi ** handle, const char *systemcfgfile)
 
   return SHISHI_OK;
 }
-
-void
-shishi_info (Shishi * handle, const char *fmt, ...)
-{
-  va_list ap;
-  va_start (ap, fmt);
-
-  fprintf (stderr, INFOSTR);
-  vfprintf (stderr, fmt, ap);
-  fprintf (stderr, "\n");
-
-  va_end (ap);
-}
-
-void
-shishi_warn (Shishi * handle, const char *fmt, ...)
-{
-  va_list ap;
-  va_start (ap, fmt);
-
-  fprintf (stderr, WARNSTR);
-  vfprintf (stderr, fmt, ap);
-  fprintf (stderr, "\n");
-
-  va_end (ap);
-}
