@@ -121,12 +121,14 @@
 
 #include "gettext.h"
 
-#include "setenv.h"
+#include "base64.h"
+#include "getdate.h"
+#include "time_r.h"
+#include "timegm.h"
 #include "vasprintf.h"
 #include "xalloc.h"
+#include "xmemdup.h"
 #include "xstrndup.h"
-#include "getdate.h"
-#include "base64.h"
 
 #include "shishi.h"
 
@@ -159,7 +161,8 @@
 #define HOST_NAME_MAX BUFSIZ
 #endif
 
-enum {
+enum
+{
   UDP,
   TCP,
   TLS
@@ -177,6 +180,8 @@ struct Shishi_realminfo
   char *name;
   struct Shishi_kdcinfo *kdcaddresses;
   size_t nkdcaddresses;
+  char **serverwildcards;
+  size_t nserverwildcards;
 };
 
 struct Shishi
@@ -238,6 +243,8 @@ int _shishi_encapreppart_input (Shishi * handle, FILE * fh,
 				Shishi_asn1 * encapreppart, int type);
 int _shishi_safe_input (Shishi * handle, FILE * fh,
 			Shishi_asn1 * safe, int type);
+int _shishi_priv_input (Shishi * handle, FILE * fh,
+			Shishi_asn1 * priv, int type);
 
 Shishi_asn1 _shishi_asn1_read (void);
 int _shishi_cipher_init (void);
@@ -250,8 +257,10 @@ shishi_asn1_integer2_field (Shishi * handle,
 extern time_t xtime (time_t * t);
 extern int xgettimeofday (struct timeval *tv, struct timezone *tz);
 
-struct Shishi_realminfo *
-shishi_realminfo (Shishi * handle, const char *realm);
+extern struct Shishi_realminfo *shishi_realminfo (Shishi * handle,
+						  const char *realm);
+extern struct Shishi_realminfo *shishi_realminfo_new (Shishi * handle,
+						      char *realm);
 
 #if defined(WITH_DMALLOC) && WITH_DMALLOC
 #include <dmalloc.h>
