@@ -43,8 +43,8 @@ shishi_as_derive_salt (Shishi * handle,
 		       Shishi_asn1 asreq,
 		       Shishi_asn1 asrep, char *salt, size_t * saltlen)
 {
-  int len = *saltlen;
-  int tmplen;
+  size_t len = *saltlen;
+  size_t tmplen;
   char *format;
   int res;
   int i, n;
@@ -89,9 +89,9 @@ shishi_as_derive_salt (Shishi * handle,
 
   for (i = 1; i <= n; i++)
     {
-      tmplen = *saltlen - len;
-      if (tmplen < 0)
+      if (*saltlen < len)
 	return SHISHI_TOO_SMALL_BUFFER;
+      tmplen = *saltlen - len;
 
       asprintf (&format, "req-body.cname.name-string.?%d", i);
       res = shishi_asn1_read (handle, asreq, format, salt + len, &tmplen);
@@ -112,11 +112,11 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
 			Shishi_asn1 * kdcrep)
 {
   char *der;
-  int der_len;
+  size_t der_len;
   size_t buflen;
   char *buffer;
   char *realm;
-  int realmlen;
+  size_t realmlen;
   int res;
 
   res = shishi_new_a2d (handle, kdcreq, &der, &der_len);
@@ -132,7 +132,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not get realm: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return res;
     }
   realm = xrealloc (realm, realmlen + 1);
@@ -142,7 +142,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not send to KDC: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return res;
     }
   free (realm);
@@ -165,7 +165,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
 		{
 		  shishi_error_printf
 		    (handle, "Could not DER decode AS-REP/KRB-ERROR: %s",
-		     shishi_strerror_details (handle));
+		     shishi_error (handle));
 		  return SHISHI_ASN1_ERROR;
 		}
 
@@ -576,8 +576,7 @@ shishi_tgs_process (Shishi * handle,
 		    Shishi_asn1 tgsreq,
 		    Shishi_asn1 tgsrep,
 		    Shishi_asn1 authenticator,
-		    Shishi_asn1 oldenckdcreppart,
-		    Shishi_asn1 * enckdcreppart)
+		    Shishi_asn1 oldenckdcreppart, Shishi_asn1 * enckdcreppart)
 {
   Shishi_key *tktkey;
   Shishi_key *subkey;
@@ -599,27 +598,39 @@ shishi_tgs_process (Shishi * handle,
     return res;
 
   if (etype != shishi_key_type (use_subkey ? subkey : tktkey))
-    return SHISHI_TGSREP_BAD_KEYTYPE;
+    res = SHISHI_TGSREP_BAD_KEYTYPE;
+  else
+    res = shishi_kdc_process (handle, tgsreq, tgsrep,
+			      use_subkey ? subkey : tktkey,
+			      use_subkey ?
+			      SHISHI_KEYUSAGE_ENCTGSREPPART_AUTHENTICATOR_KEY
+			      : SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
+			      enckdcreppart);
 
-  res = shishi_kdc_process (handle, tgsreq, tgsrep,
-			    use_subkey ? subkey : tktkey,
-			    use_subkey ?
-			    SHISHI_KEYUSAGE_ENCTGSREPPART_AUTHENTICATOR_KEY :
-			    SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
-			    enckdcreppart);
-  if (res == SHISHI_CRYPTO_ERROR && use_subkey)
+  /* Entire if statement to work around buggy KDCs. */
+  if (use_subkey && (res == SHISHI_CRYPTO_ERROR ||
+		     res == SHISHI_TGSREP_BAD_KEYTYPE))
     {
       int tmpres;
 
-      tmpres = shishi_kdc_process (handle, tgsreq, tgsrep, tktkey,
-				   SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
-				   enckdcreppart);
+      /* Try again using key from ticket instead of subkey */
+      if (etype != shishi_key_type (tktkey))
+	tmpres = SHISHI_TGSREP_BAD_KEYTYPE;
+      else
+	tmpres = shishi_kdc_process (handle, tgsreq, tgsrep, tktkey,
+				     SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
+				     enckdcreppart);
+
+      /* if bug workaround code didn't help, return original error. */
       if (tmpres != SHISHI_OK)
 	return res;
 
       shishi_warn (handle, "KDC bug: Reply encrypted using wrong key.");
+
+      res = tmpres;
     }
-  else if (res != SHISHI_OK)
+
+  if (res != SHISHI_OK)
     return res;
 
   return SHISHI_OK;
@@ -648,7 +659,7 @@ shishi_as_process (Shishi * handle,
 		   const char *string, Shishi_asn1 * enckdcreppart)
 {
   unsigned char salt[BUFSIZ];
-  int saltlen;
+  size_t saltlen;
   int res;
   Shishi_key *key;
   int keytype;
