@@ -59,7 +59,7 @@ shishi_key_principal_set (Shishi_key * key, const char *principal)
   if (key->principal)
     free (key->principal);
   if (principal)
-    key->principal = strdup (principal);
+    key->principal = xstrdup (principal);
   else
     key->principal = NULL;
 }
@@ -92,7 +92,7 @@ shishi_key_realm_set (Shishi_key * key, const char *realm)
   if (key->realm)
     free (key->realm);
   if (realm)
-    key->realm = strdup (realm);
+    key->realm = xstrdup (realm);
   else
     key->realm = NULL;
 }
@@ -212,16 +212,12 @@ shishi_key_length (Shishi_key * key)
  *
  * Create a new Key information structure.
  *
- * Return value: Returns SHISHI_MALLOC_ERROR on memory allocation
- *               errors, and SHISHI_OK on success.
+ * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_key (Shishi * handle, Shishi_key ** key)
 {
-  *key = malloc (sizeof (**key));
-  if (!*key)
-    return SHISHI_MALLOC_ERROR;
-  memset (*key, 0, sizeof (**key));
+  *key = xcalloc (1, sizeof (**key));
 
   (*key)->handle = handle;
 
@@ -232,13 +228,12 @@ shishi_key (Shishi * handle, Shishi_key ** key)
  * shishi_key_done:
  * @key: pointer to structure that holds key information.
  *
- * Deallocates key information structure and set key handle to NULL.
+ * Deallocates key information structure.
  **/
 void
-shishi_key_done (Shishi_key ** key)
+shishi_key_done (Shishi_key * key)
 {
-  free (*key);
-  *key = NULL;
+  free (key);
 }
 
 /**
@@ -269,8 +264,7 @@ shishi_key_copy (Shishi_key * dstkey, Shishi_key * srckey)
  * key value. KEY contains a newly allocated structure only if this
  * function is successful.
  *
- * Return value: Returns SHISHI_MALLOC_ERROR on memory allocation
- *               errors, and SHISHI_OK on success.
+ * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_key_from_value (Shishi * handle,
@@ -300,8 +294,7 @@ shishi_key_from_value (Shishi * handle,
  * key value. KEY contains a newly allocated structure only if this
  * function is successful.
  *
- * Return value: Returns SHISHI_MALLOC_ERROR on memory allocation
- *               errors, SHISHI_INVALID_KEY if the base64 encoded key
+ * Return value: Returns SHISHI_INVALID_KEY if the base64 encoded key
  *               length doesn't match the key type, and SHISHI_OK on
  *               success.
  **/
@@ -322,9 +315,7 @@ shishi_key_from_base64 (Shishi * handle,
       size_t len;
       char *buf;
 
-      buf = malloc (strlen (value) + 1);
-      if (!buf)
-	return SHISHI_MALLOC_ERROR;
+      buf = xmalloc (strlen (value) + 1);
 
       len = base64_from (buf, value);
 
@@ -388,8 +379,7 @@ shishi_key_random (Shishi * handle, int32_t type, Shishi_key ** key)
  * key value using shishi_random_to_key().  KEY contains a newly
  * allocated structure only if this function is successful.
  *
- * Return value: Returns SHISHI_MALLOC_ERROR on memory allocation
- *               errors, and SHISHI_OK on success.
+ * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_key_from_random (Shishi * handle,
@@ -422,8 +412,7 @@ shishi_key_from_random (Shishi * handle,
  * key value using shishi_string_to_key().  KEY contains a newly
  * allocated structure only if this function is successful.
  *
- * Return value: Returns SHISHI_MALLOC_ERROR on memory allocation
- *               errors, and SHISHI_OK on success.
+ * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_key_from_string (Shishi * handle,
