@@ -50,11 +50,8 @@ shishi_tgs (Shishi * handle, Shishi_tgs ** tgs)
   Shishi_tgs *ltgs;
   int res;
 
-  *tgs = malloc (sizeof (**tgs));
-  if (*tgs == NULL)
-    return SHISHI_MALLOC_ERROR;
+  *tgs = xcalloc (1, sizeof (**tgs));
   ltgs = *tgs;
-  memset (ltgs, 0, sizeof (*ltgs));
 
   ltgs->handle = handle;
 
@@ -62,7 +59,7 @@ shishi_tgs (Shishi * handle, Shishi_tgs ** tgs)
   if (ltgs->tgsreq == NULL)
     {
       shishi_error_printf (handle, "Could not create TGS-REQ: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -70,7 +67,7 @@ shishi_tgs (Shishi * handle, Shishi_tgs ** tgs)
   if (ltgs->tgsreq == NULL)
     {
       shishi_error_printf (handle, "Could not create TGS-REP: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -78,7 +75,7 @@ shishi_tgs (Shishi * handle, Shishi_tgs ** tgs)
   if (ltgs->krberror == NULL)
     {
       shishi_error_printf (handle, "Could not create KRB-ERROR: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -97,6 +94,25 @@ shishi_tgs (Shishi * handle, Shishi_tgs ** tgs)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_as_done:
+ * @as: structure that holds information about AS exchange
+ *
+ * Deallocate resources associated with AS exchange.  This should be
+ * called by the application when it no longer need to utilize the AS
+ * exchange handle.
+ **/
+void
+shishi_tgs_done (Shishi_tgs * tgs)
+{
+  shishi_asn1_done (tgs->handle, tgs->tgsreq);
+  shishi_asn1_done (tgs->handle, tgs->tgsrep);
+  shishi_asn1_done (tgs->handle, tgs->krberror);
+  shishi_ap_done (tgs->ap);
+  shishi_tkt_done (tgs->tkt);
+  free (tgs);
+}
+
 /**
  * shishi_tgs_tgtkt:
  * @tgs: structure that holds information about TGS exchange
@@ -177,7 +193,7 @@ shishi_tgs_req_set (Shishi_tgs * tgs, Shishi_asn1 tgsreq)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_tgs_req_der (Shishi_tgs * tgs, char **out, size_t *outlen)
+shishi_tgs_req_der (Shishi_tgs * tgs, char **out, size_t * outlen)
 {
   int rc;
 
@@ -330,7 +346,7 @@ shishi_tgs_rep (Shishi_tgs * tgs)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_tgs_rep_der (Shishi_tgs * tgs, char **out, size_t *outlen)
+shishi_tgs_rep_der (Shishi_tgs * tgs, char **out, size_t * outlen)
 {
   int rc;
 
@@ -391,12 +407,8 @@ shishi_tgs_rep_process (Shishi_tgs * tgs)
   if (VERBOSEASN1 (tgs->handle))
     shishi_ticket_print (tgs->handle, stdout, ticket);
 
+  /* XXX */
   tgs->tkt = shishi_tkt2 (tgs->handle, ticket, kdcreppart, tgs->tgsrep);
-  if (tgs->tkt == NULL)
-    {
-      shishi_error_printf (tgs->handle, "Could not create ticket");
-      return SHISHI_MALLOC_ERROR;
-    }
 
   return SHISHI_OK;
 }
@@ -482,7 +494,7 @@ shishi_tgs_krberror (Shishi_tgs * tgs)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_tgs_krberror_der (Shishi_tgs * tgs, char **out, size_t *outlen)
+shishi_tgs_krberror_der (Shishi_tgs * tgs, char **out, size_t * outlen)
 {
   int rc;
 
