@@ -91,7 +91,7 @@ shishi_tkt_build (Shishi_tkt * tkt, Shishi_key * key)
  * Return value: Returns client principal of ticket.
  **/
 int
-shishi_tkt_client (Shishi_tkt * tkt, char *client, int *clientlen)
+shishi_tkt_client (Shishi_tkt * tkt, char *client, size_t * clientlen)
 {
   return shishi_principal_name_get (tkt->handle, tkt->kdcrep,
 				    "cname", client, clientlen);
@@ -101,13 +101,11 @@ int
 shishi_tkt_client_p (Shishi_tkt * tkt, const char *client)
 {
   char *buf;
-  int buflen;
+  size_t buflen;
   int res;
 
   buflen = strlen (client) + 1;
-  buf = malloc (buflen);
-  if (buf == NULL)
-    return 0;
+  buf = xmalloc (buflen);
 
   res = shishi_tkt_client (tkt, buf, &buflen);
   if (res != SHISHI_OK)
@@ -129,7 +127,8 @@ shishi_tkt_client_p (Shishi_tkt * tkt, const char *client)
 }
 
 int
-shishi_tkt_cnamerealm (Shishi_tkt * tkt, char *cnamerealm, int *cnamerealmlen)
+shishi_tkt_cnamerealm (Shishi_tkt * tkt,
+		       char *cnamerealm, size_t * cnamerealmlen)
 {
   return shishi_principal_name_realm_get (tkt->handle,
 					  tkt->kdcrep, "cname",
@@ -141,13 +140,11 @@ int
 shishi_tkt_cnamerealm_p (Shishi_tkt * tkt, const char *client)
 {
   char *buf;
-  int buflen;
+  size_t buflen;
   int res;
 
   buflen = strlen (client) + 1;
-  buf = malloc (buflen);
-  if (buf == NULL)
-    return 0;
+  buf = xmalloc (buflen);
 
   res = shishi_tkt_cnamerealm (tkt, buf, &buflen);
   if (res != SHISHI_OK)
@@ -322,11 +319,7 @@ shishi_tkt2 (Shishi * handle,
 {
   Shishi_tkt *tkt;
 
-  tkt = malloc (sizeof (*tkt));
-  if (tkt == NULL)
-    return NULL;
-
-  memset (tkt, 0, sizeof (*tkt));
+  tkt = xcalloc (1, sizeof (*tkt));
 
   tkt->handle = handle;
   tkt->ticket = ticket;
@@ -351,10 +344,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
   Shishi_tkt *t;
   int res;
 
-  t = malloc (sizeof (*t));
-  if (t == NULL)
-    return SHISHI_MALLOC_ERROR;
-  memset (t, 0, sizeof (*t));
+  t = xcalloc (1, sizeof (*t));
 
   t->handle = handle;
 
@@ -362,7 +352,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
   if (t->ticket == NULL)
     {
       shishi_error_printf (handle, "Could not create Ticket: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -371,7 +361,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
   if (t->enckdcreppart == NULL)
     {
       shishi_error_printf (handle, "Could not create EncKDCRepPart: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -379,7 +369,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
   if (t->encticketpart == NULL)
     {
       shishi_error_printf (handle, "Could not create EncTicketPart: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -405,7 +395,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
   if (t->kdcrep == NULL)
     {
       shishi_error_printf (handle, "Could not create AS-REP: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -418,7 +408,7 @@ void
 shishi_tkt_done (Shishi_tkt * tkt)
 {
   if (tkt->key)
-    shishi_key_done (&tkt->key);
+    shishi_key_done (tkt->key);
   free (tkt);
 }
 
@@ -578,13 +568,13 @@ shishi_tkt_ok_as_delegate_p (Shishi_tkt * tkt)
 }
 
 int
-shishi_tkt_realm (Shishi_tkt * tkt, char **realm, size_t *realmlen)
+shishi_tkt_realm (Shishi_tkt * tkt, char **realm, size_t * realmlen)
 {
   return shishi_ticket_realm_get (tkt->handle, tkt->ticket, realm, realmlen);
 }
 
 int
-shishi_tkt_server (Shishi_tkt * tkt, char *server, int *serverlen)
+shishi_tkt_server (Shishi_tkt * tkt, char *server, size_t * serverlen)
 {
   return shishi_ticket_sname_get (tkt->handle, tkt->ticket,
 				  server, serverlen);
@@ -594,13 +584,11 @@ int
 shishi_tkt_server_p (Shishi_tkt * tkt, const char *server)
 {
   char *buf;
-  int buflen;
+  size_t buflen;
   int res;
 
   buflen = strlen (server) + 1;
-  buf = malloc (buflen);
-  if (buf == NULL)
-    return 0;
+  buf = xmalloc (buflen);
 
   res = shishi_tkt_server (tkt, buf, &buflen);
   if (res != SHISHI_OK)
@@ -623,7 +611,7 @@ shishi_tkt_server_p (Shishi_tkt * tkt, const char *server)
 
 int
 shishi_tkt_server_realm (Shishi_tkt * tkt,
-			 char *serverrealm, int *serverrealmlen)
+			 char *serverrealm, size_t * serverrealmlen)
 {
   return shishi_ticket_snamerealm_get (tkt->handle, tkt->ticket,
 				       serverrealm, serverrealmlen);
@@ -652,7 +640,7 @@ shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype)
 
 int
 shishi_tkt_lastreq (Shishi_tkt * tkt,
-		    char *lrtime, int *lrtimelen, int lrtype)
+		    char *lrtime, size_t * lrtimelen, int lrtype)
 {
   char *format;
   int tmplrtype;
@@ -693,7 +681,7 @@ time_t
 shishi_tkt_lastreqc (Shishi_tkt * tkt, Shishi_lrtype lrtype)
 {
   char lrtime[GENERALIZEDTIME_TIME_LEN + 1];
-  int lrtimelen;
+  size_t lrtimelen;
   time_t t;
   int res;
 
@@ -741,7 +729,7 @@ shishi_tkt_lastreq_pretty_print (Shishi_tkt * tkt, FILE * fh)
 }
 
 int
-shishi_tkt_authtime (Shishi_tkt * tkt, char *authtime, int *authtimelen)
+shishi_tkt_authtime (Shishi_tkt * tkt, char *authtime, size_t * authtimelen)
 {
   return shishi_asn1_read (tkt->handle, tkt->enckdcreppart, "authtime",
 			   authtime, authtimelen);
@@ -751,7 +739,7 @@ time_t
 shishi_tkt_authctime (Shishi_tkt * tkt)
 {
   char authtime[GENERALIZEDTIME_TIME_LEN + 1];
-  int authtimelen;
+  size_t authtimelen;
   time_t t;
   int res;
 
@@ -768,7 +756,8 @@ shishi_tkt_authctime (Shishi_tkt * tkt)
 }
 
 int
-shishi_tkt_starttime (Shishi_tkt * tkt, char *starttime, int *starttimelen)
+shishi_tkt_starttime (Shishi_tkt * tkt, char *starttime,
+		      size_t * starttimelen)
 {
   return shishi_asn1_read_optional (tkt->handle, tkt->enckdcreppart,
 				    "starttime", starttime, starttimelen);
@@ -795,7 +784,7 @@ shishi_tkt_startctime (Shishi_tkt * tkt)
 }
 
 int
-shishi_tkt_endtime (Shishi_tkt * tkt, char *endtime, int *endtimelen)
+shishi_tkt_endtime (Shishi_tkt * tkt, char *endtime, size_t * endtimelen)
 {
   return shishi_asn1_read (tkt->handle, tkt->enckdcreppart, "endtime",
 			   endtime, endtimelen);
@@ -805,7 +794,7 @@ time_t
 shishi_tkt_endctime (Shishi_tkt * tkt)
 {
   char endtime[GENERALIZEDTIME_TIME_LEN + 1];
-  int endtimelen;
+  size_t endtimelen;
   time_t t;
   int res;
 
@@ -822,7 +811,8 @@ shishi_tkt_endctime (Shishi_tkt * tkt)
 }
 
 int
-shishi_tkt_renew_till (Shishi_tkt * tkt, char *renewtill, int *renewtilllen)
+shishi_tkt_renew_till (Shishi_tkt * tkt, char *renewtill,
+		       size_t * renewtilllen)
 {
   return shishi_asn1_read_optional (tkt->handle, tkt->enckdcreppart,
 				    "renew-till", renewtill, renewtilllen);
@@ -832,7 +822,7 @@ time_t
 shishi_tkt_renew_tillc (Shishi_tkt * tkt)
 {
   char renewtill[GENERALIZEDTIME_TIME_LEN + 1];
-  int renewtilllen;
+  size_t renewtilllen;
   time_t t;
   int res;
 
@@ -872,7 +862,7 @@ shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh)
 {
   char buf[BUFSIZ];
   char *p;
-  int buflen;
+  size_t buflen;
   int keytype, etype, flags;
   int res;
   time_t t;
