@@ -90,7 +90,7 @@ shishi_tkts_default_file_set (Shishi * handle, const char *tktsfile)
   if (handle->tktsdefaultfile)
     free (handle->tktsdefaultfile);
   if (tktsfile)
-    handle->tktsdefaultfile = strdup (tktsfile);
+    handle->tktsdefaultfile = xstrdup (tktsfile);
   else
     handle->tktsdefaultfile = NULL;
 }
@@ -127,10 +127,7 @@ shishi_tkts_default_to_file (Shishi_tkts * tkts)
 int
 shishi_tkts (Shishi * handle, Shishi_tkts ** tkts)
 {
-  *tkts = malloc (sizeof (**tkts));
-  if (*tkts == NULL)
-    return SHISHI_MALLOC_ERROR;
-  memset (*tkts, 0, sizeof (**tkts));
+  *tkts = xcalloc (1, sizeof (**tkts));
 
   (*tkts)->handle = handle;
 
@@ -219,9 +216,7 @@ shishi_tkts_remove (Shishi_tkts * tkts, int ticketno)
 
   if (tkts->ntkts > 0)
     {
-      tkts->tkts = realloc (tkts->tkts, sizeof (*tkts->tkts) * tkts->ntkts);
-      if (tkts->tkts == NULL)
-	return SHISHI_MALLOC_ERROR;
+      tkts->tkts = xrealloc (tkts->tkts, sizeof (*tkts->tkts) * tkts->ntkts);
     }
   else
     {
@@ -247,11 +242,9 @@ shishi_tkts_add (Shishi_tkts * tkts, Shishi_tkt * tkt)
     return SHISHI_INVALID_TICKET;
 
   if (tkts->ntkts++ == 0)
-    tkts->tkts = malloc (sizeof (*tkts->tkts));
+    tkts->tkts = xmalloc (sizeof (*tkts->tkts));
   else
-    tkts->tkts = realloc (tkts->tkts, sizeof (*tkts->tkts) * tkts->ntkts);
-  if (tkts->tkts == NULL)
-    return SHISHI_MALLOC_ERROR;
+    tkts->tkts = xrealloc (tkts->tkts, sizeof (*tkts->tkts) * tkts->ntkts);
 
   tkts->tkts[tkts->ntkts - 1] = tkt;
 
@@ -277,9 +270,8 @@ shishi_tkts_new (Shishi_tkts * tkts,
   Shishi_tkt *tkt;
   int res;
 
+  /* XXX */
   tkt = shishi_tkt2 (tkts->handle, ticket, enckdcreppart, kdcrep);
-  if (tkt == NULL)
-    return SHISHI_MALLOC_ERROR;
 
   res = shishi_tkts_add (tkts, tkt);
   if (res != SHISHI_OK)
@@ -399,7 +391,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
 	{
 	  shishi_error_printf (tkts->handle,
 			       "Could not print ticket: %s\n",
-			       shishi_strerror_details (tkts->handle));
+			       shishi_error (tkts->handle));
 	  return res;
 	}
 
@@ -409,7 +401,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
 	{
 	  shishi_error_printf (tkts->handle,
 			       "Could not print ticket: %s\n",
-			       shishi_strerror_details (tkts->handle));
+			       shishi_error (tkts->handle));
 	  return res;
 	}
 
@@ -419,7 +411,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
 	{
 	  shishi_error_printf (tkts->handle,
 			       "Could not print ticket: %s\n",
-			       shishi_strerror_details (tkts->handle));
+			       shishi_error (tkts->handle));
 	  return res;
 	}
 
@@ -525,12 +517,7 @@ shishi_tkts_print_for_service (Shishi_tkts * tkts, FILE * fh,
 	  int buflen;
 
 	  buflen = strlen (service) + 1;
-	  buf = malloc (buflen);
-	  if (buf == NULL)
-	    {
-	      res = SHISHI_MALLOC_ERROR;
-	      goto done;
-	    }
+	  buf = xmalloc (buflen);
 
 	  res = shishi_tkt_server (tkt, buf, &buflen);
 	  if (res != SHISHI_OK)
@@ -851,7 +838,7 @@ shishi_tkts_get (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
 	  shishi_error_printf (tkts->handle,
 			       "AS exchange failed: %s\n%s\n",
 			       shishi_strerror (rc),
-			       shishi_strerror_details (tkts->handle));
+			       shishi_error (tkts->handle));
 	  if (rc == SHISHI_GOT_KRBERROR)
 	    shishi_krberror_pretty_print (tkts->handle, stdout,
 					  shishi_as_krberror (as));
@@ -893,7 +880,7 @@ shishi_tkts_get (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
   if (rc != SHISHI_OK)
     {
       printf ("TGS exchange failed: %s\n%s\n", shishi_strerror (rc),
-	      shishi_strerror_details (tkts->handle));
+	      shishi_error (tkts->handle));
       if (rc == SHISHI_GOT_KRBERROR)
 	shishi_krberror_pretty_print (tkts->handle, stdout,
 				      shishi_tgs_krberror (tgs));
@@ -904,8 +891,7 @@ shishi_tkts_get (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
 
   if (!tkt)
     {
-      printf ("No ticket in TGS-REP?!: %s\n",
-	      shishi_strerror_details (tkts->handle));
+      printf ("No ticket in TGS-REP?!: %s\n", shishi_error (tkts->handle));
       return NULL;
     }
 
