@@ -318,7 +318,7 @@ crypto (Shishi * handle, struct arguments arg)
       if (key == NULL)
 	{
 	  fprintf (stderr, _("Could not find key: %s\n"),
-		   shishi_strerror_details (handle));
+		   shishi_error (handle));
 	  return 1;
 	}
     }
@@ -881,9 +881,8 @@ static struct argp argp = {
   options,
   parse_opt,
   "[CLIENT [SERVER]] [OPTION...]\n"
-  "--list [CLIENT [SERVER]]\n"
-  "--destroy [CLIENT [SERVER]]\n"
-  "--string-to-key [CLIENT] [OPTION...]\n",
+    "--list [CLIENT [SERVER]]\n"
+    "--destroy [CLIENT [SERVER]]\n" "--string-to-key [CLIENT] [OPTION...]\n",
   "Shishi -- A Kerberos 5 implementation",
   NULL,
   NULL,
@@ -1038,7 +1037,7 @@ main (int argc, char *argv[])
       rc = crypto (handle, arg);
       if (rc != SHISHI_OK)
 	fprintf (stderr, "Operation failed:\n%s\n%s\n",
-		 shishi_strerror (rc), shishi_strerror_details (handle));
+		 shishi_strerror (rc), shishi_error (handle));
       break;
 
     case OPTION_RENEW:
@@ -1068,8 +1067,7 @@ main (int argc, char *argv[])
 	    rc = shishi_tkt_pretty_print (tkt, stdout);
 	    if (rc != SHISHI_OK)
 	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc),
-		       shishi_strerror_details (handle));
+		       shishi_strerror (rc), shishi_error (handle));
 	  }
 
 	/* Get ticket using TGT ... */
@@ -1094,7 +1092,7 @@ main (int argc, char *argv[])
 	if (rc != SHISHI_OK)
 	  {
 	    printf ("TGS exchange failed: %s\n%s\n", shishi_strerror (rc),
-		    shishi_strerror_details (handle));
+		    shishi_error (handle));
 	    if (rc == SHISHI_GOT_KRBERROR)
 	      shishi_krberror_pretty_print (handle, stdout,
 					    shishi_tgs_krberror (tgs));
@@ -1104,8 +1102,7 @@ main (int argc, char *argv[])
 	tkt = shishi_tgs_tkt (tgs);
 	if (!tkt)
 	  {
-	    printf ("No ticket in TGS-REP?!: %s\n",
-		    shishi_strerror_details (handle));
+	    printf ("No ticket in TGS-REP?!: %s\n", shishi_error (handle));
 	    break;
 	  }
 
@@ -1141,8 +1138,7 @@ main (int argc, char *argv[])
 	    rc = shishi_tkt_pretty_print (tkt, stdout);
 	    if (rc != SHISHI_OK)
 	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc),
-		       shishi_strerror_details (handle));
+		       shishi_strerror (rc), shishi_error (handle));
 	  }
       }
       break;
