@@ -393,7 +393,7 @@ asreq1 (Shishi * handle, struct arguments *arg, Shishi_as * as)
   Shishi_key *sessionkey, *userkey;
   int etype, i;
   char buf[BUFSIZ];
-  int buflen;
+  size_t buflen;
   int err;
   char *username, *servername, *realm;
 
@@ -484,7 +484,7 @@ asreq1 (Shishi * handle, struct arguments *arg, Shishi_as * as)
 
 static void
 asreq (Shishi * handle, struct arguments *arg,
-       Shishi_asn1 kdcreq, char **out, int *outlen)
+       Shishi_asn1 kdcreq, char **out, size_t *outlen)
 {
   Shishi_as *as;
   int rc;
@@ -677,7 +677,7 @@ tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
 
 static void
 tgsreq (Shishi * handle, struct arguments *arg,
-	Shishi_asn1 kdcreq, char **out, int *outlen)
+	Shishi_asn1 kdcreq, char **out, size_t *outlen)
 {
   Shishi_tgs *tgs;
   int rc;
@@ -1136,7 +1136,7 @@ setup (Shishi * handle, struct arguments *arg)
 
   rc = launch (handle, arg);
 
-  shishi_key_done (&arg->tgskey);
+  shishi_key_done (arg->tgskey);
 
   return rc;
 }
