@@ -368,7 +368,12 @@ str2key[] =
       "\x4b\x6d\x98\x39\xf8\x44\x06\xdf\x1f\x09\xcc\x16\x6d\xb4\xb8\x3c"
       "\x57\x18\x48\xb7\x84\xa3\xd6\xbd\xc3\x46\x58\x9a\x3e\x39\x3f\x9e",
       SHISHI_AES256_CTS_HMAC_SHA1_96, "\x00\x00\x00\x32"}
-,};
+  ,
+  {
+  "foo", "",
+      "\xac\x8e\x65\x7f\x83\xdf\x82\xbe\xea\x5d\x43\xbd\xaf\x78\x00\xcc",
+      SHISHI_RC4_HMAC}
+};
 
 struct pkcs5
 {
@@ -498,12 +503,12 @@ main (int argc, char *argv[])
 	res = shishi_dr (handle, key, drdk[i].usage, drdk[i].nusage,
 			 out, strlen (drdk[i].dr));
 
-      shishi_key_done (&key);
+      shishi_key_done (key);
 
       if (res != SHISHI_OK)
 	{
 	  fail ("shishi_dr() entry %d failed (%s)\n",
-		i, shishi_strerror_details (handle));
+		i, shishi_error (handle));
 	  continue;
 	}
 
@@ -557,12 +562,12 @@ main (int argc, char *argv[])
       if (res == SHISHI_OK)
 	res = shishi_dk (handle, key, drdk[i].usage, drdk[i].nusage, key2);
 
-      shishi_key_done (&key);
+      shishi_key_done (key);
 
       if (res != SHISHI_OK)
 	{
 	  fail ("shishi_dk() entry %d failed (%s)\n",
-		i, shishi_strerror_details (handle));
+		i, shishi_error (handle));
 	  continue;
 	}
 
@@ -622,7 +627,7 @@ main (int argc, char *argv[])
       if (res != SHISHI_OK)
 	{
 	  fail ("shishi_n_fold() entry %d failed (%s)\n",
-		i, shishi_strerror_details (handle));
+		i, shishi_error (handle));
 	  continue;
 	}
 
@@ -678,7 +683,7 @@ main (int argc, char *argv[])
       if (res != SHISHI_OK)
 	{
 	  fail ("shishi_string_to_key() entry %d failed (%s)\n",
-		i, shishi_strerror_details (handle));
+		i, shishi_error (handle));
 	  continue;
 	}
 
